/********************************************************************/
/*                                                                  */
/*   Program Name :  oscillo                                        */
/*                                                                  */
/*      Version:  1.00  2015-12-22                                  */
/*                                                                  */
/*      Copyright (c) 2015-     T.Kondo      All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
 
 Time domain display of VSSP data

        Ver. 2015-10-27 start development 
             2015-12-22  

    How to run:
       oscillo vssp_file [options]

        where 
               vssp_file -- VSSP32 format data file name

         [options (any order)]

              -p[mode] pmode
                    plot device selection
                        pmode=0 :  display and PostScript file out
                             =1 :  PostScript file out only
                             =2 :  display only
                             default is 2
              -t[span]  tspan  -- set time axis span (sec)
                                  default auto 
              -o[ffset] soffset -- start time offset
              -ps   --  set postscript out
              -h[alt] --  halt after each plot (default continuous plot) 
              -s[msec] smsec
                         set suspend time (msec) after each plot
                         default is 0 (as soon as posiible base)

*/
#define PROGRAM_VERSION "Ver. 2015-12-22"

#include "oscillo.h"    // header for graphics

#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#define DEFLT_DATA_FILE "tds.data"   // default data file name

//#define DEFLT_SLEEP_MSEC 100        // default sleep suspend time in msec
#define DEFLT_D_POINTS 100     // default display span in # of data

void print_usage(speana_param_t *st)
{
    char pgdevice_dflt[20];
    printf("speana %s",PROGRAM_VERSION);
#ifdef USE_GNUPLOT   /* using gnuplot */
    printf(" compliled for GNUPLOT \n");
    strcpy(pgdevice_dflt,G_DISPLAY);
#else
    printf(" compliled for PGPLOT \n");
    strcpy(pgdevice_dflt,st->g_display);
#endif
    printf("\n");
    printf("Usage: oscillo filename [options]\n");
    printf("\n");
    printf("         where  filename ---- Data file name\n");
    //printf("                              if 0 is given, tds.data will be used\n");
    printf("\n");
    printf("            [options]\n");
//    printf("                -p[mode] pmode\n");
//    printf("                     plot device selection\n");
//    printf("                        pmode=0 :  display and PostScript file (%s) out\n",st->ps_fname);
//    printf("                             =1 :  PostScript file (%s) out only\n",st->ps_fname);
//    printf("                             =2 :  display only\n");
//    printf("                            default is 2\n");
    printf("                -t[span] tspan --- display period (sec)\n");
    //printf("                -c comment --- Comment appeared on a graph\n");
    //printf("                -ps  --- post script out too\n");
    printf("                -h[alt]  --- halt after each plot (default continuous plot)\n");
    printf("                -s[msec] smsec\n");
    printf("                     set suspend time (msec) after each plot\n");
    printf("                     default is 0 (as soon as possible base)\n");

#ifndef USE_GNUPLOT   // using gnuplot
    printf("\n");
    printf("  Note: %s ---  environment variable for PGPLOT display device\n",PGDISP);
    printf("                   = %s",getenv(PGDISP));
    if(getenv(PGDISP)==NULL) {
        printf(",  program default is ( %s )\n",PGDISP_DFLT);
    } else {
        printf("\n");
    }
#endif
    printf("\n");
    exit(0);
}

void runparam_get(int argc, char *argv[],runp_t *runp)                       
{
    /* get run parameters type 1 (conventional run tyep) */

    //int n,i;
    char cbuf[256];

	// initialize parameters

    runp->fname[0]='\0';
    runp->comment[0]='\0';
    runp->soffset=0.0;
    runp->tspan=0.0;
    runp->pmode=2;
    runp->flag_halt=0;
    //runp->sleep_msec=DEFLT_SLEEP_MSEC;
    runp->sleep_msec=0;

    if (argc>=2){
        strcpy(runp->fname,*(argv+1));
    }
    if (strlen(*(argv+1))==1 && runp->fname[0]=='0'){
          strcpy(runp->fname, DEFLT_DATA_FILE);
    }
    
    if(cmnd_get(argc,argv,"-p",cbuf)==0) runp->pmode = atoi(cbuf);     // -pmode 
    if(cmnd_get(argc,argv,"-t",cbuf)==0) runp->tspan = atof(cbuf);     //  -tspan 
    if(cmnd_get(argc,argv,"-a",cbuf)==0) runp->tspan = atof(cbuf);     //  -tspan   to keep compatibility with speana_n
    if(cmnd_get(argc,argv,"-i",cbuf)==0) runp->tspan = atof(cbuf);     //  -tspan   to keep compatibility with speana_n
    if(cmnd_get(argc,argv,"-c",cbuf)==0) strcpy(runp->comment,cbuf);   //  -comment 
    if(cmnd_get(argc,argv,"-o",cbuf)==0) runp->soffset = atof(cbuf);   //  -offset 
    //if(cmnd_get(argc,argv,"-ps",cbuf)==0) runp->pmode=0;               // -ps
    if(cmnd_get(argc,argv,"-h",cbuf)==0) runp->flag_halt=1;           // -halt
    if(cmnd_get(argc,argv,"-s",cbuf)==0) runp->sleep_msec = atoi(cbuf);   //  -sleep_msec 

}

int main(int argc, char *argv[])
{
    //int nloop,sekibun;
    char cw;
    int nloop;
    int flag_last;   // 2015-10-27
    int i,imax, m, k, kaisu;
    int smode,usampl,nfft;
	int kbytes;
    int64_t bits, bytes;  // 2105-10-23
    int64_t ns_bopp;      // sample# from the 1st point of data file at begining of tsapn period
    int64_t ns_all;       // sample# from the 1st point of data file
    int numb;
    int p_usampl;     // current sample # in USAMPL
    int p_adata;      // current sample # in ADATA
    int flag_plot_done; 
    //int scount[4];
    float fsampl,tsampl,fact;
    int isoffset;     // start offset in integer second      2014-08-12
    double fsoffset;  // fractional part of start offset     2014-08-12
    double ftime;     // time in second to judge start time  2014-08-12
    int start_flag;   // flag to judge start time            2014-08-12
    double nfact;
    double tinteg,tusampl;
    double telapsed;           // to measure processing time
    double dw;
    clock_t cstart1sec, cfinish1sec;  // to calculate processing time
    char ltime1st[60];
    char cbuf[20];
    K5VSSP_file_t K5;
    speana_param_t st;
    runp_t runp;

    /* set graphic device */
    strcpy(st.ps_fname,PS_OUT_FILE);
#ifdef USE_GNUPLOT   // using gnuplot
    strcpy(st.g_display,G_DISPLAY);
    strcpy(st.g_ps_device,G_PS_DEVICE);
#else
    strcpy(st.g_display,select_env_dflt(PGDISP, PGDISP_DFLT));
    sprintf(st.g_ps_device,"%s%s",st.ps_fname,G_PS_DEVICE);
#endif

    if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){
         print_usage(&st);
         exit(0);
    }
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ 
          printf("oscillo %s",PROGRAM_VERSION);
#ifdef USE_GNUPLOT   /* using gnuplot */
          printf(" compliled for GNUPLOT \n");
#else
          printf(" compliled for PGPLOT \n");
#endif
          printf("\n");
          exit(0);
    }
    
    /* get run parameters */
    runparam_get(argc, argv, &runp);

    /* copy runparameters to other variables */
    strcpy(st.fname,runp.fname);
    strcpy(K5.fname,st.fname);

    strcpy(st.comment,runp.comment);
    st.soffset=runp.soffset;
    st.tspan=runp.tspan;
    st.toffset=runp.soffset;

    isoffset=(int)st.soffset;
    fsoffset=fmod(st.soffset,1.0);
    if(fsoffset > 0.0){
        start_flag=0;
    } else {
        start_flag=1;
    }
    ftime=0.0;

    printf("oscillo %s",PROGRAM_VERSION);
    printf("\n");

    cstart1sec = clock();

    printf("Data File is %s\n",K5.fname);
    K5.size=get_file_size(K5.fname);
    if( (K5.stream  = fopen( K5.fname, "rb" )) == NULL ){
         printf("File (%s) Open Error! \n",K5.fname);
        goto exitend2;
    }
    k5vssp_t_ini(&K5);


    /* Data header check to get sampling parameters */

    if((kbytes=CheckK5header_with_Read(&K5)) < 0){
        printf("File read error! \n");
        i=fclose( K5.stream ) ;
        goto exitend;
    }

    /* parameter set */

    st.pmode=runp.pmode;
    st.flag_halt=runp.flag_halt;
    st.sleep_msec=runp.sleep_msec;

    flag_last=0;
    st.sfreq=K5.sfreq;
    st.adbit=K5.adbit;
    st.numch=K5.numch;
    fsampl=(float)(1000.0*(float)st.sfreq);   // sampling frequency (Hz)
    tsampl=(float)(1.0/fsampl);            // sampling period (sec)
    bits=((int64_t)fsampl)*st.adbit*st.numch;  // # of bits in 1 sec data except header  2015-10-23
    bytes=bits/8;                     // # of bytes in 1 sec data except header
    K5.fsample=fsampl;
    //sekibun=1;                       // Averaging period for Spectrum (sec)
    K5.bytesec=(int)(bits/8);                  //# of bytes in 1 sec data

    smode=1;                         // low resolution mode

    /* unit sample # set to read */

    if(smode ==1){ 
        usampl=10000; 
    } else {
        usampl=50000;
        if(st.sfreq == 40)  usampl=40000; 
        if(st.sfreq == 100) usampl=100000; 
        if(st.sfreq == 200) usampl=200000; 
    }
    imax=(st.sfreq*1000)/usampl;
    numb=usampl*st.adbit*st.numch/8;   // # of bytes in unit # of samples
    K5.numb=usampl*K5.adbit*K5.numch/8;       // # of bytes in a usampl
    K5.imax=(K5.sfreq*1000)/usampl;        // # of usampls in a second


    tusampl=(double)usampl/K5.fsample;   // period of usampl

    printf("imax numb  %d %d\n",K5.imax,K5.numb);    //debug


    /* set display time span default */
    if(st.tspan==0.0){
        //st.tspan=tusampl;
        st.tspan=DEFLT_D_POINTS*tsampl;
        printf("display span set to %g(sec)\n",st.tspan);
    }

    /* set # of samples in tspan */
    dw=st.tspan*K5.fsample;
    if(dw > pow(2.0,31.0)/4.0){  //  bigger than maximum of 32-bit integer
        printf("given display span (%e sec) exceeds program limit %f\n",st.tspan,(pow(2.0,31.0)/4.0)/K5.fsample);
        printf("change parameter then run again!\n");
        i=fclose( K5.stream ) ;
        goto exitend;
    }
    st.nadata=(int)(dw+0.5);


    /* set FFT size */
    // not used in this program but to keep consistency with other programs

    nfft=1024;
    if(usampl < 524288) nfft= 524288;
    if(usampl < 262144) nfft= 262144;
    if(usampl < 131072) nfft= 131072;
    if(usampl < 65536)  nfft= 65536;
    if(usampl < 32768)  nfft= 32768;
    if(usampl < 16384)  nfft= 16384;

    st.nfft=nfft;
    st.usampl=usampl;

    nfact=2.0/(double)nfft;

    K5.nword=K5.numb/4;                 // # of 32bit-word in a usampl
    K5.tsample=1.0/K5.fsample;    // 2015-10-23
    K5.unitp=(double)usampl*K5.tsample;   // period in a usampl (sec)

    K5.ubuf  = (u_int32_t *)malloc(K5.nword*sizeof(u_int32_t)); // K5 data read buffer
    K5.idata  = (int *)malloc(32*K5.nword*sizeof(int)); // K5 data read buffer

    //st.xdata = (float *)malloc(nfft*sizeof(float));
    //st.adata = (float *)malloc(4*nfft*sizeof(float));
    //st.adata = (float *)malloc(4*usampl*sizeof(float));
    st.adata = (float *)malloc(4*st.nadata*sizeof(float));

    /* foward startffset seconds */
    //if(fwdNsecK5(st.soffset, &K5) <0){
    if(fwdNsecK5(isoffset, &K5) <0){ 
        //printf("Foward error %d\n",st.soffset);
        printf("Foward error %d\n",isoffset);
        i=fclose( K5.stream ) ;
        goto exitend;
    }

    printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
        K5.hh,K5.mm,K5.ss,K5.seconds);
    if(K5.format==2){  // VSSP32          2014-06/13
	   sprintf(ltime1st,"Time(UT) %04d/%03d %02d:%02d:%02d",
           K5.year,K5.totalday,K5.hh,K5.mm,K5.ss);
    } else {
       sprintf(ltime1st,"Time(UT) %02d:%02d:%02d",K5.hh,K5.mm,K5.ss);
    }
    if(fsoffset > 0.0){   // 2014-08-12
        sprintf(cbuf,".%03d",(int)(1000.0*fsoffset));
        strcat(ltime1st,cbuf);
    }
    strcpy(st.lab2,ltime1st);
    strcat(st.lab2,"  ");
    strcat(st.lab2,st.comment);
    
        
    flag_plot_done=0;
    tinteg=0.0;
    p_usampl=0;
    p_adata=0;
    // initialize array
    for(k=0; k<4*st.nadata; k++) st.adata[k]=0.0;

    st.flag_g_device_open=0;

    ns_all=0;

    nloop=0;
loop1:
    if(nloop >0) {
        if((kbytes=CheckK5header_with_Read(&K5)) < 0){
            i=fclose( K5.stream ) ;
            goto exitend;
        }
        printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
            K5.hh,K5.mm,K5.ss,K5.seconds);
    }


    for(i=0; i<K5.imax; i++){
        K5.wordread = fread(K5.ubuf, sizeof(u_int32_t), K5.nword, K5.stream);
        if(K5.wordread < K5.nword){  // probably no more data     2015-10-27
            flag_last=1;
            if(K5.wordread==0){
                tusampl=0;
                goto exithere;
            }
            /* recalc tusampl and fact */
            if(st.adbit*st.numch >= 8){
                usampl=K5.wordread*4/(st.adbit*st.numch/8);
                fact=((float)usampl)*((float)usampl);
                tusampl=(double)usampl/K5.fsample;   // period of usampl
            } else {
                tusampl=0;
                goto exithere;
            }
        }

        K5.pos+=K5.wordread*4;
        K5.vflag=1; // validiate data
        parseADdata_3N(&K5);

        if(start_flag==0){
            if(ftime < fsoffset){
                ftime+=tusampl;
                continue;
            } else {
                start_flag=1;
            }
        }

        /* check current sample # */
        //p_usampl+=K5.npos/K5.numch;



        for(k=0; k<K5.npos/K5.numch; k++){
            kaisu=0;
            for(m=0; m<st.numch; m++){
                /* set data */
                if(p_adata < st.nadata){
                    if(p_adata==0 && m==0){  // keep time offset of begining of adata
                        ns_bopp=ns_all;
                        st.toffset=(double)ns_bopp*K5.tsample;
                    }
                    st.adata[p_adata*4+m]=(float)K5.idata[k*st.numch+m];


                } else {
                    // plot here
                    plottime(&st);
                    for(i=0; i<4*st.nadata; i++) st.adata[i]=0.0;
                    flag_plot_done=1;
                    p_adata=0;
                    p_usampl=k;
                    //st.toffset+=st.tspan;
                    goto break1;
                }
                kaisu++;
            }

            p_adata++;
            ns_all++;
break1:;
            if(flag_plot_done==1){
                if(st.flag_halt==1){
                   //printf(" Hit return key for next span (CTRL-C for stop)  ");
                   printf(" Hit return key for next span (or \"N\"on-stop or \"Q\"uit) ");
                   cw=getchar();  /* dummy read to accept just return */
                   if(cw=='N' || cw=='n') st.flag_halt=0;
                   if(cw=='Q' || cw=='q') goto exitloop;
                } else {
                   spe_sleep(st.sleep_msec);
                }
                flag_plot_done=0;
            }
        }

exithere:
        tinteg+=tusampl;

        if(flag_last == 1){
            if(tinteg==0.0) goto exitloop;
            plottime(&st);
            goto exitloop;  
        }

    }
    goto loop1;
exitloop:
    i=fclose( K5.stream ) ;

    cfinish1sec = clock();
    telapsed = (double)(cfinish1sec - cstart1sec) / CLOCKS_PER_SEC;
    //printf("Time elapsed for processing is %f sec\n",telapsed);

  exitend: ;
      
    free(K5.ubuf);
    free(K5.idata);
    free(st.adata);

 exitend2: ;

    return 0;
}
