/*********************************************************************/
/*                                                                   */
/*   Program Name :  sampling                                        */
/*                                                                   */
/*      Version:  0.00  2001-08-17                                   */
/*                1.00  2002-05-26                                   */
/*                1.10  2004-05-23                                   */
/*                1.11  2005-06-12  minor update                     */
/*                1.12  2005-06-16  Alow big file by command         */
/*                1.13  2005-09-28   support linux2.6                */
/*                2.00  2005-12-28  support VSSP32                   */
/*                2.01  2006-01-02  change due to AUX field change   */
/*                2.02  2007-03-15  add independent LPF set          */
/*                2.03  2007-08-09  debug LPF set in case of         */
/*                                    Fsample < 1 MHz                */
/*                2.04  2007-08-23  add bitshift set parameter       */
/*                2.05  2008-02-19  options '--help' and '--version' */
/*                                  were added                       */
/*                2.06  2011-02-09  system(...) was changed to like  */
/*                                  i=system(...) to avoid warning   */
/*                                  at Ubuntu 10.10                  */
/*                2.07   2015-01-21 remove '\0' in sprintf           */
/*                2.10   2016-05-13 change default file creation to  */
/*                                   a big file                      */
/*                                  support VSSP64                   */
/*                                                                   */
/*      Copyright (c) 2002-2016 T.Kondo/NICT All Right Reserved      */
/*                                                                   */
/*********************************************************************/
/*

 off line sampling program for VSSP32/64

         by T.Kondo    2001.8.17
                1.12   2005-06-16  Alow big file by command
                1.13   2005-09-28  support linux2.6
                2.00   2005-12-21  support VSSP32
                2.02   2007-03-15  add independent LPF set parameter
                2.03   2007-08-09  debug LPF set in case of Fsample < 1 MHz
                2.04   2007-08-23  add bitshift set parametert
                2.05   2008-02-19  options '--help' and '--version' were added
                2.06   2011-02-09  system(...) was changed to like i=system(...) 
                                   to avoid warning at Ubuntu 10.10
                2.07   2015-01-21  remove '\0' in sprintf
                2.10   2016-05-18  change default file creation to a big file
                                   and support VSSP64

    How to run:
       sampling span sfreq[:lpf] adbit[:bitshift] numch [filename [logfile]]
   
        where  span   sampling period (sec)
                      -VE means small file (2GB/file)
               sfreq  sampling frequency (in MHz)
                       0.04,0.1,0.2,0.5,1,2,4,8,16           for VSSP
                       0.04,0.1,0.2,0.5,1,2,4,8,16,32,64     for VSSP32
                       2,4,8,16,32,64,128                    for VSSP64
                       -VE means no use of digital filter
                       +VE LPF filter is set as follows (valid for VSSP32/64)
                       for VSSP32
                         Fsample(MHz)   0.04 0.1 0.2 0.5 1 2 4 8 16 32 64
                         Ffilter(MHz)     2   2   2   2  2 2 2 4  8 16  0
                                                   (0 means through)
                       for VSSP64
                         Fsample(MHz)   2 4 8 16 32 64 128
                         Ffilter(MHz)   2 2 4  8 16 32   0
   
                       lpf    set LPF (MHz)  2,4,8,16,32,0   (0 means throughj
                         use this option when u want to set LPF independenly
                                                   
                adbit  set AD resolution
                       1,2,4,8 
                bitshift  set bitshit value in case of 2bit or 4bit AD resolution
                        0-6 can be set (default is 0)
                numch  set # of channels
                       1,2,4       2 is only for VSSP64
                filename -- data file name to be created
                             default is "tds.data"
                logfile -- log file name
                      default is "sampling.log"
                      default file will be overwritten.
                      when user set log file name, log will be appended
*/
#define SAMPLING_VERSION "Ver. 2016-05-18"

#include "chead.h"      // common header for IP-VLBI utilities

#include "libipvlbi_board.h"  // IP-VLBITv[{[hp֐Q(by T.Kondo)

#define DFLT_FNAME "tds.data"           // default data file name
#define DFLT_LOGFILE "sampling.log"     // default log file name
                        
//int g_model=2;            // VSSP model   0: VSSP 1: VSSP32  2: VSSP64


typedef struct run_param    /* run parameter structure */
{            
   char outfile[60];
   char logfile[80];
   int span;
   int sfreq;
   int filter;
   int adbit;
   int bitshift;
   int numch;
   int logmode;       // 0: log file is first cleared
   int f_size_mode;   // 1: divided into 2GB size file  0: no limitation
   int aux_fmt;       //  default AUX field format for K5/VSSP32 (1: autoobs  2: sampling)
                      // 21: # of Ch =2 (in case of VSSP64)
   double freq;
  

} run_param_t;

void print_usage(void)
{
    printf("\n");
    printf("sampling %s\n",SAMPLING_VERSION);
    printf("      compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
    printf("\n");
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    printf("Usage: sampling span sfreq[:lpf] adbit[:bitshift] numch [filename [logfile]]\n");
#else
    printf("Usage: sampling span sfreq adbit numch [filename [logfile]]\n");
#endif
    printf("\n");
    printf("         where  span ---- duration (sec)\n");
    //printf("                          (-VE means allow big file)\n");
    printf("                          (-VE : files are devided into 2GB a file)\n");  // 2016-05-13
    //printf("                          (otherwise devided into 2GB files)\n");
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    printf("                sfreq ---- sampling frequency\n");
   #ifdef _VSSP32
    printf("                           40,100,200,500 (for kHz)\n");
    printf("                           0.04,0.1,0.2,0.5,1,2,4,8,16,32,64 (for MHz)\n");
   #else
    printf("                           2,4,8,16,32,64 (MHz)\n");
   #endif
    printf("                     Note: -VE means ""no use of digital filter"" at K5/VSSP32\n");
    printf("                           In case of +VE, relation between sampling frequency\n");
    printf("                            (Fs) and filter frequency(Ff) is as follows;\n");
   #ifdef _VSSP32
    printf("                            Fs(MHz)   0.04 0.1 0.2 0.5 1 2 4 8 16 32 64\n");
    printf("                            Ff(MHz)     2   2   2   2  2 2 2 4  8 16  0\n");
   #else
    printf("                            Fs(MHz)   2 4 8 16 32 64 128\n");
    printf("                            Ff(MHz)   2 2 4  8 16 32   0\n");
   #endif
    printf("                                   (0 means through)\n");
    printf("                lpf   ---- set LPF frequency when u want to set idependently\n");
    printf("                           from sampling frequency\n");
   #ifdef _VSSP32
    printf("                             2,4,8,16,0 (MHz)\n");
   #else
    printf("                             2,4,8,16,32,0 (MHz)\n");
   #endif
    printf("                             (0 means through)\n");
#else
    printf("                sfreq ---- sampling frequency\n");
    printf("                           40,100,200,500 (for kHz)\n");
    printf("                           1,2,4,8,16 (for MHz)\n");
#endif
    printf("                adbit ---- A/D bit\n");
    printf("                           1,2,4,8\n");
    printf("                           default is 1\n");
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    printf("                bitshift ---- # of bits to be shifted from 8 bit sampling data\n");
    printf("                           in case of 2bit or 4 bit sampling.\n");
    printf("                           0-6 (default is 0)\n");
#endif
    printf("                numch ---- number of channel\n");
#ifdef _VSSP64
    printf("                           1,2,4\n");
#else
    printf("                           1,4\n");
#endif
    printf("                           default is 1\n");
    printf("                filename ---- out file name\n");
    printf("                           default is %s\n",DFLT_FNAME);
    printf("                logfile ---- log out file name\n");
    printf("                           default is %s\n",DFLT_LOGFILE);
    printf("                           if default logfile is used, it is overwritten.\n");
    printf("                           But if logfile is set by ""logfile"" parameter,\n");
    printf("                           log will be appended to logfile.\n");
}


void parse_sfreq_lpf(char *cbuf, int *sfreq, int *lpf, double *freq)
{
//    parse "sfreq[:lpf]"  to sfreq and lpf
    int sfreq0,lpf0,n;
	int no_filter;
    double freq0;
    char cset[]=":";
    char *p= NULL;

    freq0=0.0;
    sfreq0=0;
    lpf0=0;
	n=0;

	freq0=atof(strtok(cbuf,cset));
    sfreq0=(int)freq0;
	while((p=strtok(NULL,cset))){
		n++;
		if(n==1) lpf0=atoi(p);
	}

    no_filter=0;
    if(freq0 < 0.0) {
        freq0=-freq0;
        no_filter=1;
	}
    sfreq0=(int)freq0;
    if(sfreq0==0) sfreq0=(int)(freq0*1000.0);  // when freq is less than 1.0

	if(n==0){    // no ":"  set LPF automatically 
       if(no_filter == 0) {
          switch(sfreq0){  // 2008-08-09
             case  4: lpf0=2; break;
             case  8: lpf0=4; break;
             case 16: lpf0=8; break;
             case 32: lpf0=16; break;
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
   #ifdef _VSSP32
             case 64: lpf0=0; break;   // VSSP32
   #else
             case 64: lpf0=32; break;   // VSSP64
             case 128: lpf0=0; break;   // VSSP64
   #endif
#endif
             default: lpf0=2; // default is minimum settable filter frequency (MHz)
          }          
	   }
	} else {
		// LPF range chaeck
#ifdef _VSSP64
		if(lpf0 < 2 || lpf0 > 32) lpf0=0;
#else
		if(lpf0 < 2 || lpf0 > 16) lpf0=0;
#endif
	}
	if(freq0 > 1.0){
      switch(sfreq0){  // 2008-08-09
         case   40: freq0=0.04; break;
         case  100: freq0=0.1 ; break;
         case  200: freq0=0.2 ; break;
         case  500: freq0=0.5 ; break;
      }
    }
    *sfreq=sfreq0;
    *lpf=lpf0;
    *freq=freq0;
}

void parse_adbit_bitshift(char *cbuf, int *adbit, int *bitshift)
{
//    parse "adbit[:bitshift]"  to adbit and bitshift
    int adbit0,bitshift0,n;
    char cset[]=":";
    char *p= NULL;

    adbit0=1;
    bitshift0=0;
	n=0;

	adbit0=atoi(strtok(cbuf,cset));
	while((p=strtok(NULL,cset))){
		n++;
		if(n==1) bitshift0=atoi(p);
	}
    *adbit=adbit0;
    *bitshift=bitshift0;
}



void runparam_get(int argc, char *argv[],run_param_t *runp)
{
    /* get run parameters */

   char cbuf[100];

   // initialize
   runp->span=0;
   runp->numch=1;
   runp->adbit=1;
   runp->bitshift=0;
   runp->aux_fmt=2;  // default AUX field format for K5/VSSP32  (sampling)
   runp->logmode=0;  // log file is first cleared
   runp->f_size_mode=0;    //1: divided into 2GB size file  0: no limitation
   strcpy(runp->outfile,DFLT_FNAME);
   strcpy(runp->logfile,DFLT_LOGFILE);
   
   if (argc>=2){
      runp->span = atoi(*(argv + 1 ));
      if(runp->span < 0 ){
         runp->span=-runp->span;
         runp->f_size_mode=1;
      }
   }
   if (argc>=3){
      strcpy(cbuf,*(argv+2));
      //parse_sfreq_lpf(cbuf, &sfreq, &spg.sp.filter, &freq);    // check ":" is in characters
      parse_sfreq_lpf(cbuf, &runp->sfreq, &runp->filter, &runp->freq);    // check ":" is in characters
   }
   if (argc>=4){
      strcpy(cbuf,*(argv+3));
      //parse_adbit_bitshift(cbuf, &adbit, &spg.sp.bitshift);    // check ":" is in characters
      parse_adbit_bitshift(cbuf, &runp->adbit, &runp->bitshift);    // check ":" is in characters
   }

   if (argc>=5){
      runp->numch = atoi(*(argv + 4 ));
      if(runp->numch==2) runp->aux_fmt=21;  // use extended format
   }

   if (argc>=6){
      strcpy(runp->outfile,*(argv+5));
   }
   if (argc>=7){
      strcpy(runp->logfile,*(argv+6));
      runp->logmode=1; // log is appended to existed log file
   }
   
}

void vssp_aux_make2(vsspaux_t *vsspaux)  // based on vssp_aux_make in libipvlbi.c
{
/*  generate AUX field data
      in cluding Fmt# 21 and 22
*/
   
    int n,len;
    char cbuf1[8],cbuf2[8];
    /* fill space exceeding 8char portion */
    for(n=0; n<8; n++){
        cbuf1[n]=' ';
        cbuf2[n]=' ';
    }
    len=strlen(vsspaux->station8);
    for(n=0; n<8 && n<len; n++) cbuf1[n]=vsspaux->station8[n];
    len=strlen(vsspaux->hostname8);
    for(n=0; n<8 && n<len; n++) cbuf2[n]=vsspaux->hostname8[n];
    
    /* format field set */
    vsspaux->bdat[0]= (char)(vsspaux->fmt & 0xFF);
    /* filter field set */
    vsspaux->bdat[1]= (char)(vsspaux->filter & 0xFF);
    /* aux field set */
    if(vsspaux->fmt==1){
       /* station field set */
       //vsspaux->bdat[1]=vsspaux->sid[0];
       vsspaux->bdat[2]=vsspaux->sid2[0];
       vsspaux->bdat[3]=vsspaux->sid2[1];
       for(n=0; n<8; n++) vsspaux->bdat[4+n]=cbuf1[n];
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
    } else if(vsspaux->fmt==2){
       /* fill out data with 0x55 */
       for(n=0; n<10; n++) vsspaux->bdat[2+n]= 0x55;
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
    } else if(vsspaux->fmt==85){
       /* fill out all data with 0x55 */
       for(n=0; n<18; n++) vsspaux->bdat[2+n]= 0x55;
    } else if(vsspaux->fmt==170){
       /* fill out all data with 0xAA */
       for(n=0; n<18; n++) vsspaux->bdat[2+n]= (char)0xAA;
    } else if(vsspaux->fmt==21){
       /* fill out all data with 0xAA */
       for(n=0; n<18; n++) vsspaux->bdat[2+n]= (char)0xAA;
       /* set extra sampling frequency field and #CH field */
       vsspaux->bdat[2]= 0x1;     // means # of CH = 2
       vsspaux->bdat[3]= 0; 
    } else {
       /* format 0 is adapted */
       vsspaux->fmt=0;
       /* fill out all data with 0x00 */
       for(n=0; n<20; n++) vsspaux->bdat[n]= 0x00;
    }
}


int main(int argc, char **argv)
{
    char outfile[60];
    char cbuf[100];
    char* dflt_fname="tds.data";
    char* dflt_logfile="sampling.log";
    char logfile[80];
    //char dev_file[20];
    int rv;
    int span, sfreq, adbit, bitshift, numch;
    int logmode;
    int idummy; // 2011-02-09
    double freq;
    run_param_t runp;
    spget_t spg;
    logval_t lg;
    //char vsspmodel[3][10]={"VSSP  ","VSSP32","VSSP64"};

    //if(argc != 5 && argc !=6 && argc !=7){
    if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
       printf("\n sampling %s\n",SAMPLING_VERSION);
       printf("      compiled for K5/%s sampler\n",VSSP_MODEL_NAME);
       printf("\n");
       exit(0);
    } 
    
    if((argc != 5 && argc !=6 && argc !=7) ||(argc == 2 && strncmp(*(argv+1),"--help",6)==0 ) ){
       print_usage();
       exit(0);
    }

    //spg.sp.vsspaux.fmt=2;  // default AUX field format for K5/VSSP32  (sampling)
    //spg.sp.vsspaux.fmt=1;  // default AUX field format for K5/VSSP32  (autoobs)

    // get run parameters

    runparam_get(argc, argv, &runp);

    // get device file name
    g_vssp_model=VSSP_MODEL;
    get_vssp_drv_name(g_vssp_model, g_dev_file);    // get device file name
    printf(" Sampling: Device file for %s => %s\n",VSSP_MODEL_NAME,g_dev_file);

    spg.sp.vsspaux.fmt=runp.aux_fmt;
    spg.f_size_mode=runp.f_size_mode;

    logmode=0;  // log file is first cleared
    //spg.f_size_mode=1;    //1: divided into 2GB size file  0: no limitation
    spg.f_size_mode=runp.f_size_mode;    //1: divided into 2GB size file  0: no limitation
    spg.sp.filter=runp.filter;
    spg.sp.bitshift=runp.bitshift;

    /* log info set if necessary */
    strcpy(lg.logfile,runp.logfile);
    lg.hostname[0]='\0';
    lg.logdir[0]='\0';
    //lg.logfiletype=-1;    //-1: no log   0: log out
    lg.logfiletype=0;    //-1: no log   0: log out
    spg.lg=lg;
    if(runp.logmode==0){
       sprintf(cbuf,"rm -f %s",runp.logfile);
       //system(cbuf);    // delete logfile
       idummy=system(cbuf);    // delete logfile  2011-02-09
    }

    printf(" span    = %d\n",runp.span);    // debug
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    printf(" sfreq   = %d (freq(MHz) = %f)\n",runp.sfreq,runp.freq);   // debug
    if(spg.sp.filter==0){
       printf(" filter  = through\n");   // debug
    } else {
       printf(" filter  = %d (MHz)\n",spg.sp.filter);   // debug
    }
#else
    printf(" sfreq   = %d\n",runp.sfreq);   // debug
#endif
    printf(" adbit   = %d\n",runp.adbit);   // debug
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    printf(" bitshift= %d\n",spg.sp.bitshift);   // debug
#endif
    printf(" numch   = %d\n",runp.numch);   // debug
    printf(" outfile = %s\n",runp.outfile);   // debug
    if(lg.logfiletype==0){
    printf(" logfile = %s\n",runp.logfile);   // debug
    }
    sprintf(cbuf,"sampling start: span=%d sfreq=%d adbit=%d numch=%d outfile=%s",
                 runp.span,runp.sfreq,runp.adbit,runp.numch,runp.outfile);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    
//#ifdef _VSSP32
#if defined(_VSSP32) || defined(_VSSP64)        // 2016-05-18
    /* prepare AUX field for K5/VSSP32 */
    /* get host name for AUX filed */
    
    hostget(spg.sp.vsspaux.hostname8);
    strcpy(spg.sp.vsspaux.sid," ");
    strcpy(spg.sp.vsspaux.sid2,"  ");
    strcpy(spg.sp.vsspaux.station8,"sampling");
    spg.sp.vsspaux.size=20;  // this is fixed
    spg.sp.vsspaux.filter=spg.sp.filter;
    //vssp_aux_make(&spg.sp.vsspaux);
    vssp_aux_make2(&spg.sp.vsspaux);
#endif

    spg.span=runp.span;
    spg.sp.sfreq=runp.sfreq;
    spg.sp.adbit=runp.adbit;
    spg.sp.numch=runp.numch;
    strcpy(spg.outfile,runp.outfile);
    //spg.f_size_mode=1;    //1: divided into 2GB size file  0: no limitation
    
    rv=sampleget(&spg);
    sprintf(cbuf,"sampling stop");
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    if(rv > 0) {
       sprintf(cbuf,"read error occured at least once (%d)",rv);
       LogWrite4(&lg, 0, 0.0, 1, cbuf);
    }
    printf("Data File is %s\n",spg.outfile);
    if(lg.logfiletype==0){
       printf("Log  file is %s\n",spg.lg.logfile);
    }

    return 0;
}
